C  Program:  DumpData_1_08.f
C
C  Functions:  Accept program arguments from Standard Input (SI).
C  Open a BUFR Files and an output file "<BUFR Files>.txt".
C  Select whether to Dump All message Types or Only Levels Data.
C  Read BUFR and Create Output File.
C
C---------------------------------------------------------------------- 
C.KRK       
*
*  Version:     1.08
*   14/11/05/LJG -- revised entry of program arguments to standard input.
*  Version:     1.07
*   07/28/04/KRK -- add command-line use of user filename, mode, 43, 44
*  Version:     1.06
*   09/24/03/KRK -- revise RATP (add 87)
*  Version:     1.05
*   07/23/03/KRK -- revise RGRSY (1500C)
*  Version:     1.04
*   05/06/03/KRK -- revised RATP (both were 55) and RGRSY
*  Version:     1.03
*   04/22/03/KRK -- revised RTERM 9-5 and 9-17 (for B.T. numbering)
*  Version:     1.02
*   04/14/03/KRK -- revised RTERM 9-17 (bump codes by 1)
*  Version:     1.01
*   11/27/02/KRK -- expands the table lookups to text
*  Version:     1.0
*   02/14/02/KRK -- original, from Jack Woollen's example
*
C.KRK    
C---------------------------------------------------------------------- 
      program dumpdata

c     use DFLIB
      PARAMETER (ALL_DATASETS=0)
      PARAMETER (LEVELS_ONLY=1)

      LOGICAL	SawFirstLevel, exists, doloop, getFN, getDT
      COMMON /FLAGS/ SawFirstLevel

      character*1 dmptyp, eoj
      character*8 subset
      character*256 buf,InputFN,OutputFN
      integer*2 status,Mode,lenin
C
C     DEACTIVATED FOLLOWING CODE WHICH ACCEPTED PROGRAMS ARGUMENTS
C     FROM COMMAND LINE ENTRIES
C
CC     get command-line's input filename, if provided by user
C      call getarg(1,buf,status)
C      if (status .eq. -1) then
CC        none: use historical defaults
C         InputFN  = 'rrs.bfr'
C         OutputFN = 'rrs.out'
C      else
CC        create an OutputFN as the InputFN.txt; find rightmost .
C         InputFN = trim(buf)
C         ndx = index(InputFN,'.',.true.)
C  
C         if (ndx .eq. 0) then
C            OutputFN = InputFN(1:len_trim(InputFN)) // '.txt'
C         else
CC           distinguish . suffix from . or .. prefixes
C            if (InputFN(1:1) .ne. '.' .and. ndx .gt. 1) then
C               OutputFN = InputFN(1:ndx-1) // '.txt'
C            else
CC              this is very simplistic, ignoring pathological filenames
C               if (ndx .le. 2) then
C                  OutputFN = InputFN(1:len_trim(InputFN)) // '.txt'
C               else
C                  OutputFN = InputFN(1:ndx-1) // '.txt'
C               endif
C            endif
C         endif
C      endif
C
CC     get command-line's mode, if provided by user
C      call getarg(2,buf,status)
C      if (status .eq. -1) then
CC        none: use levels-only
C         Mode = LEVELS_ONLY
C         print *,"Levels only"
C      else
C         Mode = ALL_DATASETS
C         print *,"All datasets -- be patient"
C      endif
C -----------------------------------------------------
C
C    Accept Program Arguments From Standard Input -
C    1)  The Input Filename, and
C    2)  An option for Dump Type.
C
      write (6, 20)
 20   format(/' Create a formatted dump of RRS BUFR data:'
     & //' There are two dump formats:'
     &  /' 1)  Full verbose dump of all seven (7) message types',
     &   ' (Filesize = Approx 100 MB)'
     &  /' 2)  Dump Meta-data and Levels Data ')
C
      getFN = .true.
      do while (getFN)
       do i=1, 256
        InputFN(i:i) = ' '
       enddo
C
       write (6, 30)
 30    format(/' Enter BUFR Input (path & filename):')
       read (5,'(a)') InputFN
C
C  DETERMINE EXACT SIZE OF INPUT FILENAME..
       doloop = .true.
       lenin = 256
       do while (doloop .and. lenin.gt.1)
        if (InputFN(lenin:lenin) .ne. ' ') then
         doloop = .false.
        else
         lenin = lenin - 1
        endif
       enddo
C
       inquire(file=InputFN(1:lenin), exist=exists)
       if (.not. exists) then
        write(6, 40) InputFN(1:lenin)
 40     format(/' ***ERROR:  BUFR Input File Not Found:  '/a
     &     /' Please Re-Enter -'/)
       else
C
        getFN = .false.
       endif
C
       OutputFN = InputFN(1:lenin) // '.txt'
       write(6, 45) OutputFN(1:lenin+4)
 45    format(/' Output File:  ',a)
C
      enddo
C
      getDT = .true.
C
      do while (getDT)
       write(6, 50)
 50    format(/' Enter Code For Dump Type:  0 = All Message Types'
     &        /'                            1 = Levels Data Only')
       read(5, '(a1)')dmptyp 
C
       if (dmptyp .eq. '0') then     
         Mode = ALL_DATASETS
         print *,"All datasets -- be patient"
         getDT = .false.
       elseif (dmptyp .eq. '1') then
         Mode = LEVELS_ONLY
         print *,"Levels only"
         getDT = .false.
       endif
      enddo
C
      SawFirstLevel = .false.

      open( 8,file=InputFN ,form='unformatted')
      open(50,file=OutputFN,form='formatted')

      WRITE(50,*) 'Dump of Input File: ',trim(InputFN)

      call openbf(8,'IN',8)
      do while(ireadmg(8,subset,idate).eq.0)
      do while(ireadsb(8).eq.0)
         call mydump(8,50,Mode)
      enddo
      enddo

      close(50)
C
      write(6, 110)
 110  format(/' - Successful End, Press  ENTER  To End Program')
      read (5, '(a)') eoj
      stop
      end
C
      SUBROUTINE MYDUMP(LUNIT,LUPRT,Mode)
                                                                        
      PARAMETER (MAXJL=15000)
      PARAMETER (NF=2)
      PARAMETER (ALL_DATASETS=0)
      PARAMETER (LEVELS_ONLY=1)
      PARAMETER (MAXARR=100)
      REAL       Kelvin
      PARAMETER (Kelvin=273.15)

      LOGICAL	SawFirstLevel
      COMMON /FLAGS/ SawFirstLevel
 
      COMMON /MSGCWD/ NMSG(NF),NSUB(NF),MSUB(NF),INODE(NF),IDATE(NF)    
      COMMON /TABLES/ MAXTAB,NTAB,TAG(MAXJL),TYP(MAXJL),KNT(MAXJL),     
     .                JUMP(MAXJL),LINK(MAXJL),JMPB(MAXJL),              
     .                IBT(MAXJL),IRF(MAXJL),ISC(MAXJL),                 
     .                ITP(MAXJL),VALI(MAXJL),KNTI(MAXJL),               
     .                ISEQ(MAXJL,2),JSEQ(MAXJL)                         
      COMMON /USRINT/ NVAL(NF),INV(MAXJL,NF),VAL(MAXJL,NF)              
      COMMON /TABABD/ NTBA(0:NF),NTBB(0:NF),NTBD(0:NF),MTAB(50,NF),
     .                IDNA(50,NF,2),IDNB(250,NF),IDND(250,NF),
     .                TABA(50,NF),TABB(250,NF),TABD(250,NF)
 
      CHARACTER*600 TABD
      CHARACTER*128 TABB
      CHARACTER*128 TABA

      CHARACTER*80 FMT
      CHARACTER*64 DESC
      CHARACTER*24 UNIT
      CHARACTER*20 LCHR
      CHARACTER*10 TAG,NEMO                                             
      CHARACTER*6  NUMB                                                 
      CHARACTER*8  CVAL,PMISS                                            
      CHARACTER*3  TYP
      CHARACTER*1  TAB
      CHARACTER*40 CODEVAL
      EQUIVALENCE  (RVAL,CVAL)
      REAL*8       VAL,RVAL,BMISS

      CHARACTER*8  SUBSET
      REAL*8       ARR(MAXARR)
      character*10 DataSet
      integer*2    Mode
      integer      ReleaseDate,LevelDate
      integer      YR0,MO0,DY0
      integer      YR1,MO1,DY1
      real*8       HR0,MI0,SC0,ReleaseTime
      real*8       HR1,MI1,SC1,LevelTime,ET
      real*8       LVL,P,RH,TDB,TDP,GPH,WS,WD
                                                                        
      DATA BMISS /   10E10  /                                                
      DATA PMISS /' MISSING'/                                            
                                                                        
C---------------------------------------------------------------------- 
C---------------------------------------------------------------------- 
                                                                        
      if(luprt.eq.0) luout = 6                                          
      if(luprt.ne.0) luout = luprt                                      
                                                                        
C  CHECK THE FILE STATUS AND I-NODE                                     
C  --------------------------------                                     
                                                                        
      CALL STATUS(LUNIT,LUN,IL,IM)                                      
      IF(IL.EQ.0) GOTO 900                                              
      IF(IM.EQ.0) GOTO 901                                              
      IF(INODE(LUN).NE.INV(1,LUN)) GOTO 902                             

      NEMO = TAG(INODE(LUN))
      DataSet = NEMO
      SUBSET  = NEMO

C     User's Mode determines which datasets to dump
C     (Note: LEVELS_ONLY includes Metadata as well)
      IF     (NEMO.EQ.'NC002019') THEN
		Mode = Mode
      ELSEIF (NEMO.EQ.'NC002020') THEN
		if (Mode.ne.ALL_DATASETS) return
      ELSEIF (NEMO.EQ.'NC002021') THEN
		if (Mode.ne.ALL_DATASETS) return
      ELSEIF (NEMO.EQ.'NC002022') THEN
		if (Mode.ne.ALL_DATASETS) return
      ELSEIF (NEMO.EQ.'NC002023') THEN
		if (Mode.ne.ALL_DATASETS) return
      ELSEIF (NEMO.EQ.'NC002024') THEN
		if (Mode.ne.ALL_DATASETS) return
      ELSEIF (NEMO.EQ.'NC002025') THEN
		Mode = Mode
      ELSE
		return
      ENDIF

      IF     (NEMO.EQ.'NC002019') THEN
			DESC = 'RRS metadata'
      ELSEIF (NEMO.EQ.'NC002020') THEN
			DESC = 'RRS raw PTU'
      ELSEIF (NEMO.EQ.'NC002021') THEN
			DESC = 'RRS raw GPS unsmoothed'
      ELSEIF (NEMO.EQ.'NC002022') THEN
			DESC = 'RRS raw GPS smoothed'
      ELSEIF (NEMO.EQ.'NC002023') THEN
			DESC = 'RRS processed PTU'
      ELSEIF (NEMO.EQ.'NC002024') THEN
			DESC = 'RRS processed GPS'
      ELSEIF (NEMO.EQ.'NC002025') THEN
			DESC = 'RRS standard & significant levels'
      ELSE
			DESC = '?'
      ENDIF

      if (Mode.eq.ALL_DATASETS   .or. 
     *    (Mode.eq.LEVELS_ONLY   .and. 
     *	   .not. SawFirstLevel)
     *   ) then

         WRITE(LUOUT,*) 
         WRITE(LUOUT,*) 'MESSAGE TYPE ',trim(NEMO),' -- ',trim(DESC)
         WRITE(LUOUT,*) 
      endif

C  DUMP THE CONTENTS OF COMMON /USRINT/ FOR UNIT LUNIT                  
C  ---------------------------------------------------                  

      if (Mode.eq.LEVELS_ONLY .and. DataSet.eq.'NC002025') then
C	compress from one-line-per-field to just one-line-per-level
	call UFBSEQ(8,ARR,MAXARR,1,IRET,SUBSET)

			! WMOB           69.  WMO BLOCK NUMBER                        
			! WMOS            3.  WMO STATION NUMBER                      
			! WBAN        93734.  WBAN NUMBER                             
			! RATP           87.  RADIOSONDE TYPE                         9-1   Sippican GPS presumed = new
			! DATSIG          3.  DATA SIGNIFICANCE                       9-21  balloon launch point
	YR0   = ARR( 6)	! YEAR         2004.  YEAR                                    
	MO0   = ARR( 7)	! MNTH            1.  MONTH                                   
	DY0   = ARR( 8)	! DAYS           12.  DAY                                     
	HR0   = ARR( 9)	! HOUR           17.  HOUR                                    
	MI0   = ARR(10)	! MINU            5.  MINUTE                                  
	SC0   = ARR(11)	! SECO         22.89  SECOND                                  
			! CLATH     38.98000  LATITUDE (HIGH ACCURACY)                
			! CLONH    -77.48000  LONGITUDE (HIGH ACCURACY)               
			! HBMSL         85.0  HEIGHT OF BAROMETER ABOVE MSL           
			! HEIT           75.  HEIGHT                                  
			! DATSIG          6.  DATA SIGNIFICANCE                       9-21  flight level obs
	YR1   = ARR(17)	! YEAR         2004.  YEAR                                    
	MO1   = ARR(18)	! MNTH            1.  MONTH                                   
	DY1   = ARR(19)	! DAYS           12.  DAY                                     
	HR1   = ARR(20)	! HOUR           17.  HOUR                                    
	MI1   = ARR(21)	! MINU            5.  MINUTE                                  
	SC1   = ARR(22)	! SECO         22.89  SECOND                                  
	LVL   = ARR(23)	! LEVSIG         20.  LEVEL SIGNIFICANCE FOR SOUNDING         9-18  surface
			! FLPC           94.  FLIGHT LEVEL PRESSURE CORRECTION        9-11: Adj O.chk P.chk C.chk Inter
	P     = ARR(25)	! PRLC       101325.  PRESSURE                                
	RH    = ARR(26)	! REHU           5.0  RELATIVE HUMIDITY                       
			! SIRC            5.  SOLAR AND INFRARED RADIATION CORRECTIO  9-13  solar corrected
	TDB   = ARR(28)	! TMDB        298.15  TEMPERATURE/DRY BULB TEMPERATURE        
	TDP   = ARR(29)	! TMDP        255.89  DEW-POINT TEMPERATURE                   
	GPH   = ARR(30)	! GPH10          75.  GEOPOTENTIAL HEIGHT                     
			! HGHT       MISSING  HEIGHT                                  
	WS    = ARR(32)	! WSPD           5.1  WIND SPEED                              
	WD    = ARR(33)	! WDIR          270.  WIND DIRECTION                          

	ReleaseDate = NthDay(YR0,MO0,DY0)
	ReleaseTime = HR0*3600 + MI0*60 + SC0
	LevelDate   = NthDay(YR1,MO1,DY1)
	LevelTime   = HR1*3600 + MI1*60 + SC1

	ET = (LevelDate-ReleaseDate)*24*3600 + (LevelTime-ReleaseTime)	! seconds

        IDUM = MYLKUP ('LEVSIG    ',LVL,CODEVAL)

	if (.not. SawFirstLevel) then
	   WRITE(LUOUT,'(A)') '   ET        P       GPH   Tdb      RH'//
     *	                      '    Tdp      WD    WS   Level Type'
	   WRITE(LUOUT,'()')
     	endif
        FMT = '(F8.2,2x,F7.2,2x,F6.0,1x,F7.2,2x,F5.1,2x,F6.2,'//
     *	      '	2x,F4.0,2x,F5.1,2x,F3.0,2x,A)'
        WRITE(LUOUT,FMT) ET,P/100,GPH,TDB-Kelvin,RH,TDP-Kelvin,WD,WS,
     *		LVL,trim(CODEVAL(7:25))

	SawFirstLevel = .true.
	return
      endif

      if (DataSet.eq.'NC002025') SawFirstLevel = .true.

C     normal one-line-per-field dump
                                                                        
      DO NV=1,NVAL(LUN)                                                 
         NODE = INV (NV,LUN)                                                 
         NEMO = TAG (NODE)                                                   
         ITYP = ITP (NODE)                                                   
         IF(ITYP.GE.1.AND.ITYP.LE.3) THEN                      
            CALL NEMTAB(LUN,NEMO,IDN,TAB,N)
            IF(TAB.NE.'B') CALL BORT('MYDUMP - BAD ITYP!')
            NUMB = TABB(N,LUN)(1:6)
            DESC = TABB(N,LUN)(16:70)
            UNIT = TABB(N,LUN)(71:94)
            RVAL = VAL(NV,LUN)                                                 
            IDUM = MYLKUP (NEMO,RVAL,CODEVAL)
         ENDIF

         IF(ITYP.EQ.1.OR.ITYP.EQ.2) THEN                      
            IF(RVAL.LT.BMISS) THEN
	       if (DataSet.eq.'NC002019') then
                 FMT = '(A6,2X,F20.00,2X,A38,2X,A)'
	       else
                 FMT = '(A6,2X,F10.00,2X,A38,2X,A)'
	       endif
               WRITE(FMT(12:13),'(I2)') MAX(0,ISC(NODE))
               WRITE(LUOUT,FMT) NEMO,RVAL,DESC,trim(CODEVAL)
            ELSE
	       if (DataSet.eq.'NC002019') then
                 FMT = '(A6,2X,A20,2X,A38,2X,A)'
	       else
                 FMT = '(A6,2X,A10,2X,A38,2X,A)'
	       endif
               WRITE(LUOUT,FMT) NEMO,PMISS,DESC,trim(CODEVAL)
            ENDIF
         ELSEIF(ITYP.EQ.3) THEN                                            
            LCHR = ' '
            NCHR = IBT(NODE)/8
            IF(NCHR.GT.8) THEN
               CALL READLC(LUNIT,LCHR,NEMO)
            ELSE
               LCHR = CVAL
            ENDIF
            IRET = RJUST(LCHR)
	    if (DataSet.eq.'NC002019') then
               FMT = '(A6,2X,A20,2X,A38)'
	    else
               FMT = '(A6,2X,A10,2X,A38)'
	    endif
            WRITE(LUOUT,FMT) NEMO,LCHR,DESC
         ENDIF                                                             
      ENDDO                                                             
                                                                        
C  EXITS                                                                
C  -----                                                                
                                                                        
      RETURN                                                            
900   CALL BORT('MYDUMP - FILE IS CLOSED                     ')        
901   CALL BORT('MYDUMP - NO MESSAGE OPEN                    ')        
902   CALL BORT('MYDUMP - I-NODE MISMATCH                    ')        
      END                                                               
C
      FUNCTION MYLKUP(NEMO,RVAL,RESULT)
      CHARACTER*10  NEMO
      REAL*8        RVAL
      CHARACTER*40  RESULT
      INTEGER       IVAL

      MYLKUP = 0
      RESULT = ' '
      IVAL = RVAL

      IF     (NEMO.EQ.'RATP') THEN
         IF     (RVAL.EQ.55) THEN 
			RESULT = '9-1   Sippican GPS presumed = old'
         ELSEIF (RVAL.EQ.87) THEN 
			RESULT = '9-1   Sippican GPS presumed = new'
         ELSEIF (RVAL.EQ.56) THEN 
			RESULT = '9-1   InterMet GPS presumed'
         ELSE        
	                RESULT = '9-1   ? unknown or Vaisala'
         ENDIF
      ELSEIF (NEMO.EQ.'PSENS') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-2   capacitance aneroid'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-2   derived from GPS'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-2   resistive strain gage'
         ELSE        
	                RESULT = '9-2   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'TSENS') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-3   rod thermistor'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-3   bead thermistor'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-3   capacitance bead'
	 ELSE        
			RESULT = '9-3   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'RHSENS') THEN
         IF     (RVAL.EQ. 1) THEN 
			RESULT = '9-4   VIZ Mark II carbon hygristor'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-4   Vaisala H-Humicap'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-4   Capacitance sensor'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-4   Sippican Mark IIA carbon hyg'
	 ELSE        
			RESULT = '9-4   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'RGRSY') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-5   TRS-2000'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-5   IMS-1500C'
	 ELSE        
			RESULT = '9-5   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'TTSS') THEN
         IF     (RVAL.EQ. 2) THEN 
			RESULT = '9-6   RDF'
         ELSEIF (RVAL.EQ. 8) THEN 
			RESULT = '9-6   GPS'
	 ELSE        
			RESULT = '9-6   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'SFEQP') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-7   PDB'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-7   RSOIS'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-7   ASOS'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-7   Psychrometer'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-7   F420'
	 ELSE        
			RESULT = '9-7   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'BSHEL') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-8   High bay'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-8   Low bay'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-8   BILS'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-8   Roof-top BILS'
	 ELSE        
			RESULT = '9-8   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'BMFGR') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-9   Kaysam'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-9   Totex'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-9   KKS'
	 ELSE        
			RESULT = '9-9   ?'
         ENDIF
      ELSEIF (NEMO.EQ.'BTYPE') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-10a GP26'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-10a GP28'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-10a GP30'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-10a HM26'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-10a HM28'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-10a HM30'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-10a SV16'
	 ELSE        
			RESULT = '9-10a ?'
         ENDIF
      ELSEIF (NEMO.EQ.'BGTYP') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-10b hydrogen'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-10b helium'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-10b natural gas'
	 ELSE        
			RESULT = '9-10b ?'
         ENDIF
      ELSEIF (NEMO.EQ.'RCONF') THEN
         IF (IVAL.EQ.31) THEN
			RESULT = '9-10c ?'
	  ELSE
			RESULT = '9-10c'
            IF  (BTEST(IVAL,4)) THEN 
			RESULT = trim(RESULT) // ' TrReg'
	     ENDIF
            IF  (BTEST(IVAL,3)) THEN 
			RESULT = trim(RESULT) // ' LtUnit'
	     ENDIF
            IF  (BTEST(IVAL,2)) THEN 
			RESULT = trim(RESULT) // ' Parachute'
	     ENDIF
            IF  (BTEST(IVAL,1)) THEN 
			RESULT = trim(RESULT) // ' RooftopRel'
	     ENDIF
	  ENDIF
      ELSEIF (NEMO.EQ.'FLPC') THEN
         IF (IVAL.EQ.255) THEN
			RESULT = '9-11  ?'
	  ELSE
			RESULT = '9-11: '
            IF  (BTEST(IVAL,7)) THEN 
			RESULT = trim(RESULT) // ' Sm'
	     ENDIF
            IF  (BTEST(IVAL,6)) THEN 
			RESULT = trim(RESULT) // ' Adj'
	     ENDIF
            IF  (BTEST(IVAL,5)) THEN 
			RESULT = trim(RESULT) // ' Norm'
	     ENDIF
            IF  (BTEST(IVAL,4)) THEN 
			RESULT = trim(RESULT) // ' O.chk'
	     ENDIF
            IF  (BTEST(IVAL,3)) THEN 
			RESULT = trim(RESULT) // ' P.chk'
	     ENDIF
            IF  (BTEST(IVAL,2)) THEN 
			RESULT = trim(RESULT) // ' C.chk'
	     ENDIF
            IF  (BTEST(IVAL,1)) THEN 
			RESULT = trim(RESULT) // ' Inter'
	     ENDIF
	  ENDIF
      ELSEIF (NEMO.EQ.'SIRC') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-13  no correction'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-13  solar & infrared corrected'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-13  solar corrected'
	 ELSE        
			RESULT = '9-13  ?'
         ENDIF
      ELSEIF (NEMO.EQ.'RTERM') THEN
         IF     (RVAL.EQ. 1) THEN 
			RESULT = '9-17  balloon burst'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-17  balloon icing'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-17  floating balloon'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-17  weak signal'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-17  battery'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-17  ground equipment or sw'
         ELSEIF (RVAL.EQ. 7) THEN 
			RESULT = '9-17  signal interference'
         ELSEIF (RVAL.EQ. 8) THEN 
			RESULT = '9-17  radiosonde failure'
         ELSEIF (RVAL.EQ. 9) THEN 
			RESULT = '9-17  excessive missing data'
         ELSEIF (RVAL.EQ.10) THEN 
			RESULT = '9-17  ?'
         ELSEIF (RVAL.EQ.11) THEN 
			RESULT = '9-17  excessive missing P'
         ELSEIF (RVAL.EQ.12) THEN 
			RESULT = '9-17  excessive missing T'
         ELSEIF (RVAL.EQ.13) THEN 
			RESULT = '9-17  user terminated'
	 ELSE        
			RESULT = '9-17  ?'
         ENDIF
      ELSEIF (NEMO.EQ.'LEVSIG') THEN
         IF     (RVAL.EQ. 1) THEN 
			RESULT = '9-18  within 20hPa of surface'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-18  P < 10hPa'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-18  base pressure for SSI'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-18  begin missing data'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-18  begin missing RH'
         ELSEIF (RVAL.EQ. 7) THEN 
			RESULT = '9-18  begin missing T'
         ELSEIF (RVAL.EQ. 8) THEN 
			RESULT = '9-18  highest level before descent'
         ELSEIF (RVAL.EQ.10) THEN 
			RESULT = '9-18  end missing data'
         ELSEIF (RVAL.EQ.11) THEN 
			RESULT = '9-18  end missing RH'
         ELSEIF (RVAL.EQ.12) THEN 
			RESULT = '9-18  end missing T'
         ELSEIF (RVAL.EQ.13) THEN 
			RESULT = '9-18  freezing level'
         ELSEIF (RVAL.EQ.14) THEN 
			RESULT = '9-18  standard P'
         ELSEIF (RVAL.EQ.15) THEN 
			RESULT = '9-18  operator added level'
         ELSEIF (RVAL.EQ.16) THEN 
			RESULT = '9-18  operator deleted level'
         ELSEIF (RVAL.EQ.17) THEN 
			RESULT = '9-18  balloon re-ascended'
         ELSEIF (RVAL.EQ.18) THEN 
			RESULT = '9-18  sig RH'
         ELSEIF (RVAL.EQ.20) THEN 
			RESULT = '9-18  surface'
         ELSEIF (RVAL.EQ.21) THEN 
			RESULT = '9-18  sig T'
         ELSEIF (RVAL.EQ.23) THEN 
			RESULT = '9-18  flight termination'
         ELSEIF (RVAL.EQ.24) THEN 
			RESULT = '9-18  tropopause'
         ELSEIF (RVAL.EQ.28) THEN 
			RESULT = '9-18  sig wind'
         ELSEIF (RVAL.EQ.29) THEN 
			RESULT = '9-18  max wind'
         ELSEIF (RVAL.EQ.30) THEN 
			RESULT = '9-18  fixed regional wind'
         ELSEIF (RVAL.EQ.32) THEN 
			RESULT = '9-18  wind termination'
         ELSEIF (RVAL.EQ.33) THEN 
			RESULT = '9-18  P 100-110hPa'
         ELSEIF (RVAL.EQ.40) THEN 
			RESULT = '9-18  inversion'
         ELSEIF (RVAL.EQ.41) THEN 
			RESULT = '9-18  sig RH (NCDC)'
         ELSEIF (RVAL.EQ.42) THEN 
			RESULT = '9-18  sig T  (NCDC)'
         ELSEIF (RVAL.EQ.43) THEN 
			RESULT = '9-18  begin missing wind'
         ELSEIF (RVAL.EQ.44) THEN 
			RESULT = '9-18  end missing wind'
	 ELSE        
			RESULT = '9-18  ?'
         ENDIF
      ELSEIF (NEMO.EQ.'MAQC') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-19  pass, not manually checked'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-19  pass, manual pass'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-19  pass, manually deleted'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-19  fail, not manually checked'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-19  fail, manually deleted'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-19  fail, manually re-inserted'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-19  questionable, not manually checked'
         ELSEIF (RVAL.EQ. 7) THEN 
			RESULT = '9-19  questionable, manually deleted'
         ELSEIF (RVAL.LT.15) THEN 
			RESULT = '9-19  ?'
	 ELSE        
			RESULT = '9-19  missing data'
         ENDIF
      ELSEIF (NEMO.EQ.'QCCHEK') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-20  passed all checks'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-20  missing data'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-20  descend/reascend balloon check'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-20  plausibility (above limit) check'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-20  plausibility (below limit) check'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-20  superadiabatic lapse rate check'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-20  limiting angles check'
         ELSEIF (RVAL.EQ. 7) THEN 
			RESULT = '9-20  ascension rate check'
         ELSEIF (RVAL.EQ. 8) THEN 
			RESULT = '9-20  excessive change from prev flight check'
         ELSEIF (RVAL.EQ. 9) THEN 
			RESULT = '9-20  balloon overhead check'
         ELSEIF (RVAL.EQ.10) THEN 
			RESULT = '9-20  wind speed check'
         ELSEIF (RVAL.EQ.11) THEN 
			RESULT = '9-20  wind direction check'
         ELSEIF (RVAL.EQ.12) THEN 
			RESULT = '9-20  dependency/inconsistency check'
         ELSEIF (RVAL.EQ.13) THEN 
			RESULT = '9-20  data valid but modified flag'
         ELSEIF (RVAL.EQ.14) THEN 
			RESULT = '9-20  outlier check'
	 ELSE        
			RESULT = '9-20  ?'
         ENDIF
      ELSEIF (NEMO.EQ.'DATSIG') THEN
         IF     (RVAL.EQ. 0) THEN 
			RESULT = '9-21  parent/WFO site'
         ELSEIF (RVAL.EQ. 1) THEN 
			RESULT = '9-21  RRS obs site'
         ELSEIF (RVAL.EQ. 2) THEN 
			RESULT = '9-21  balloon manufacture date'
         ELSEIF (RVAL.EQ. 3) THEN 
			RESULT = '9-21  balloon launch point'
         ELSEIF (RVAL.EQ. 4) THEN 
			RESULT = '9-21  surface obs'
         ELSEIF (RVAL.EQ. 5) THEN 
			RESULT = '9-21  surface obs displacement'
         ELSEIF (RVAL.EQ. 6) THEN 
			RESULT = '9-21  flight level obs'
         ELSEIF (RVAL.EQ. 7) THEN 
			RESULT = '9-21  flight level termination'
	 ELSE        
			RESULT = '9-21  ?'
         ENDIF
      ENDIF

      RETURN
      END
C
C     determine nth day since BaseYear, for comparing dates
      FUNCTION NthDay(IY,IM,ID)
      INTEGER	      IY,IM,ID
      INTEGER	      NY,ND

      INTEGER	      DpW,DpY,DpLY,DpLP,MpY,YpLP,YpC,YpQC,BaseYear,Julian
      PARAMETER (DpW	 =   7)
      PARAMETER (DpY	 = 365)
      PARAMETER (DpLY	 = 366)
      PARAMETER (DpLP	 = (DpY + DpY + DpY + DpLY))
      PARAMETER (MpY	 =  12)
      PARAMETER (YpLP	 =   4)
      PARAMETER (YpC	 = 100)
      PARAMETER (YpQC	 = 400)

      PARAMETER (BaseYear=1600)		! a QuadCentury !

      INTEGER   JulianM(12)		! index by month: 1..12
      DATA      JulianM /0,31,59,90,120,151,181,212,243,273,304,334/

c     LOGICAL   Century,QuadCentury,OffCentury,LeapYear
      LOGICAL   Century(2100),QuadCentury(2100),OffCentury(2100),
     &          LeapYear(2100)
      Century(IY)	 = (MOD(IY,  YpC) .eq. 0)
      QuadCentury(IY)	 = (MOD(IY, YpQC) .eq. 0)
      OffCentury(IY)	 = (Century(IY) .and. .not. QuadCentury(IY))	 ! 100,200,300
      LeapYear(IY)	 = (MOD(IY, YpLP) .eq. 0 .and. .not. OffCentury(IY))

      NthDay = 0

	NY = IY - BaseYear
	ND = NY*365 + ((NY+3)/YpLP) + JulianM(IM) + ID-1;

	! correct for current year, after February
C       if (IM .gt. 2  .and.  LeapYear(IY))  ND=ND+1

	! correct for prior OffCenturies, and retain prior QuadCenturies
	ND = ND - ((NY+99)/YpC) + ((NY+399)/YpQC)

      NthDay = ND

      RETURN
      END
